<?php

namespace Drupal\mollie_commerce\EventSubscriber;

use Drupal\Core\Url;
use Drupal\mollie\Entity\Payment;
use Drupal\mollie\Events\MollieTransactionStatusChangeEvent;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpKernel\HttpKernelInterface;

/**
 * Reacts to Mollie transaction status changes.
 */
class MollieTransactionEventSubscriber extends MollieEventSubscriberBase implements EventSubscriberInterface {

  /**
   * Updates the status of the order when the status of a payment has changed.
   *
   * @param \Drupal\mollie\Events\MollieTransactionStatusChangeEvent $event
   *   The transaction status change event.
   */
  public function updateOrderStatus(MollieTransactionStatusChangeEvent $event): void {
    // Return if the event context is not this module.
    if ($event->getContext() !== 'mollie_commerce') {
      return;
    }

    // Return if the transaction is not a payment.
    if (!($event->getTransaction() instanceof Payment)) {
      return;
    }

    // We do a sub request to Drupal Commerce to handle payment and order
    // status updates there.
    $url = Url::fromRoute(
      'commerce_payment.notify',
      [
        'commerce_payment_gateway' => $this->getCommercePaymentGatewayIdByOrderId($event->getContextId()),
      ]
    );
    $sub_request = Request::create(
      $url->toString(),
      'POST',
      [
        'mollie_transaction_id' => $event->getTransaction()->getTransactionId(),
        'order_id' => $event->getContextId()
      ]
    );
    try {
      $response = $this->httpKernel
        ->handle($sub_request, HttpKernelInterface::SUB_REQUEST);
      $http_status_code = $response->getStatusCode();
    }
    catch (\Exception $e) {
      watchdog_exception('mollie', $e);
      $http_status_code = 500;
      $event->setHttpStatusCode($http_status_code);
    }

    // Set the HTTP code to return to Mollie.
    $event->setHttpStatusCode($http_status_code);
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    return [
      // Our priority should be lower than 100 to be sure the payment status is
      // updated before the order status.
      MollieTransactionStatusChangeEvent::EVENT_NAME => ['updateOrderStatus', 50],
    ];
  }

}
