<?php

/**
 * @file
 * Post update hooks for Mollie for Drupal.
 */

use Drupal\Core\Entity\EntityStorageException;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\Core\TypedData\Exception\MissingDataException;
use Drupal\Core\Utility\UpdateException;
use Drupal\mollie\Entity\Payment;

/**
 * Add a local (SQL) storage for (Mollie) payment entities.
 * 
 * We do this in a post update hook because hooks and events will be triggered
 * then.
 */
function mollie_post_update_introduce_local_sql_storage() {
  $definition_update_manager = \Drupal::entityDefinitionUpdateManager();
  $entity_type = $definition_update_manager->getEntityType('mollie_payment');

  // As our data is stored remotely and Drupal seems to be very bad at handling
  // a change from a remote storage to a local storage, we need to uninstall the
  // entity type first. We will reinstall it later.
  $definition_update_manager->uninstallEntityType($entity_type);

  // Update the entity type definition.
  $handlers = $entity_type->getHandlerClasses();
  // We need to explicitly set the handler here, although we ignore it in the
  // entity type annotation. This is because in the process of updating the
  // entity type different methods are used to determine the storage class and
  // some of them fail without the next line of code.
  $handlers['storage'] = 'Drupal\Core\Entity\Sql\SqlContentEntityStorage';
  $entity_type->set('handlers', $handlers);
  $entity_type->set('base_table', 'mollie_payment');
  $entity_keys = $entity_type->getKeys();
  $entity_keys['id'] = 'id';
  $entity_keys['label'] = 'transaction_id';
  $entity_keys['uuid'] = 'uuid';
  $entity_type->set('entity_keys', $entity_keys);

  $field_storage_definitions['id'] = BaseFieldDefinition::create('integer')
    ->setLabel(new TranslatableMarkup('ID'))
    ->setReadOnly(TRUE)
    ->setSetting('unsigned', TRUE);

  $field_storage_definitions['uuid'] = BaseFieldDefinition::create('uuid')
    ->setLabel(new TranslatableMarkup('UUID'))
    ->setReadOnly(TRUE);

  $field_storage_definitions['transaction_id'] = BaseFieldDefinition::create('string')
    ->setLabel(new TranslatableMarkup('Transaction ID'))
    ->setDescription(new TranslatableMarkup('The ID of the Mollie payment.'))
    ->setReadOnly(TRUE)
    ->addConstraint('UniqueField');

  $field_storage_definitions['context'] = BaseFieldDefinition::create('string')
    ->setLabel(new TranslatableMarkup('Context'))
    ->setDescription(new TranslatableMarkup('Type of context that requested the transaction.'));

  $field_storage_definitions['context_id'] = BaseFieldDefinition::create('string')
    ->setLabel(new TranslatableMarkup('Context ID'))
    ->setDescription(new TranslatableMarkup('ID of the context that requested the transaction.'));

  $field_storage_definitions['data'] = BaseFieldDefinition::create('map')
    ->setLabel(new TranslatableMarkup('Data'))
    ->setDescription(new TranslatableMarkup('The data related to the Mollie payment.'));

  // Now we can reinstall the entity type.
  $definition_update_manager->installFieldableEntityType($entity_type, $field_storage_definitions);

  return t('The Mollie payment entity type has been updated with local (SQL) storage.');
}

/**
 * Add a local (SQL) storage for (Mollie) payment entities.
 *
 * We do this in a post update hook because hooks and events will be triggered
 * then.
 */
function mollie_post_update_migrate_to_local_sql_storage(&$sandbox) {
  // Before running this post update hook, make sure that the Mollie payment
  // entity type has been updated with local (SQL) storage.
  if (!isset($sandbox['storage_updated'])) {
    $update_list = \Drupal::keyValue('post_update')->get('existing_updates', []);
    if (!in_array('mollie_post_update_introduce_local_sql_storage', $update_list)) {
      throw new UpdateException('The Mollie payment entity type has not yet been updated with local (SQL) storage.');
    }
    $sandbox['storage_updated'] = TRUE;
  }

  // Initialize the migration.
  if (!isset($sandbox['progress'])) {
    // Fetch the payments from the Mollie API. Mollie returns them newest first,
    // so we cannot fetch them in the iterations of the post update hook.
    /** @var \Mollie\Api\Resources\PaymentCollection $mollie_payments */
    $mollie_payments = \Drupal::service('mollie.mollie')->getClient()->payments->page();
    $sandbox['mollie_payments'] = $mollie_payments->getArrayCopy();
    while ($mollie_payments->hasNext()) {
      $mollie_payments = $mollie_payments->next();
      $sandbox['mollie_payments'] = array_merge($mollie_payments->getArrayCopy(), $sandbox['mollie_payments']);
    }
    // Sort the payments so that we can create them in Drupal in the same order
    // as they were created in the Mollie API.
    usort($sandbox['mollie_payments'], function ($a, $b) {
      return $a->createdAt <=> $b->createdAt;
    });
    $sandbox['total'] = count($sandbox['mollie_payments']);
    $sandbox['progress'] = 0;
  }

  /** @var \Mollie\Api\Resources\Payment[] $mollie_payments */
  $mollie_payments = array_slice($sandbox['mollie_payments'], 0, 25, TRUE);

  foreach ($mollie_payments as $key => $mollie_payment) {
    try {
      $data = Payment::MolliePaymentToArray($mollie_payment);
      $payment = \Drupal::entityTypeManager()->getStorage('mollie_payment')->create([
        'transaction_id' => $data['id'],
        'context' => $data['context'],
        'context_id' => $data['contextId'],
        'data' => $data,
      ]);
      $payment->save();
    }
    catch (EntityStorageException $e) {
      watchdog_exception('mollie', $e);
    }

    $sandbox['progress']++;
    unset($sandbox['mollie_payments'][$key]);
  }

  $sandbox['#finished'] = empty($sandbox['total']) ? 1 : ($sandbox['progress'] / $sandbox['total']);
  return t("Imported @progress out of @total Mollie payments. There are @todo Mollie payments left for processing.", [
    '@progress' => $sandbox['progress'],
    '@total' => $sandbox['total'],
    '@todo' => count($sandbox['mollie_payments']),
  ]);
}
