<?php

namespace Drupal\mollie\Controller;

use Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException;
use Drupal\Component\Plugin\Exception\PluginNotFoundException;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Url;
use Drupal\mollie\Events\MollieTransactionChargebackEvent;
use Drupal\mollie\Events\MollieTransactionRefundEvent;
use Drupal\mollie\Events\MollieTransactionStatusChangeEvent;
use Drupal\mollie\Mollie;
use Mollie\Api\Exceptions\ApiException;
use Mollie\Api\Types\PaymentStatus;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Response;

/**
 * Provides a controller for the webhook used by Mollie to report changes.
 */
class WebhookController extends ControllerBase {

  /**
   * Request stack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * Event dispatcher.
   *
   * @var \Symfony\Component\EventDispatcher\EventDispatcherInterface
   */
  protected $eventDispatcher;

  /**
   * Mollie connector.
   *
   * @var \Drupal\mollie\Mollie
   */
  protected $mollie;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = new static();
    $instance->requestStack = $container->get('request_stack');
    $instance->eventDispatcher = $container->get('event_dispatcher');
    $instance->mollie = $container->get('mollie.mollie');
    return $instance;
  }

  /**
   * Allows modules to react to payment status updates.
   *
   * This webhook is called by Mollie when the status of a payment changes. Once
   * the payment reaches the status 'paid' we will change the webhook to invoke
   * invokeAftercareHook() on future calls.
   *
   * @param string $context
   *   The type of context that requested the payment.
   * @param string $context_id
   *   The ID of the context that requested the payment.
   *
   * @return \Symfony\Component\HttpFoundation\Response
   *   Response that will inform Mollie whether the event was processed
   *   correctly.
   */
  public function invokeStatusChangeHook(string $context, string $context_id): Response {
    $transaction_id = $this->requestStack->getCurrentRequest()->get('id');

    // Register invocation of the webhook.
    $this->keyValue(Mollie::LAST_WEBHOOK_INVOCATION_COLLECTION_KEY)->set($transaction_id, date('c'));

    try {
      // Get the transaction for which the hook was invoked.
      /** @var \Drupal\mollie\TransactionInterface $transaction */
      $transactions = $this->entityTypeManager()->getStorage('mollie_payment')
        ->loadByProperties([
          'transaction_id' => $transaction_id,
          'context' => $context,
          'context_id' => $context_id,
        ]);
      // The transaction ID should be unique, so we can safely assume that there
      // is only one transaction with the given ID.
      $transaction = reset($transactions);

      if (!$transaction) {
        // Just return a 200 OK if the transaction could not be loaded. This is
        // advised by Mollie.
        return new Response('', 200);
      }

      // Get the updated payment from Mollie.
      $payment = $this->mollie->getClient()->payments->get($transaction->getTransactionId());

      // If the payment is paid, we change the webhook to start "aftercare".
      if (PaymentStatus::STATUS_PAID === $payment->status) {
        // Set the new webhook URL.
        $payment->webhookUrl = Url::fromRoute(
          'mollie.webhook.aftercare',
          ['context' => $context, 'context_id' => $context_id]
        )->setAbsolute()->toString();

        // Change webhook when using an alternative base URL in test mode.
        $config = $this->config('mollie.config');
        if ($config->get('test_mode') && $config->get('webhook_base_url') !== '') {
          $defaultBaseUrl = Url::fromRoute('<front>')
            ->setAbsolute()->toString();
          $payment->webhookUrl = str_replace(
            $defaultBaseUrl,
            "{$config->get('webhook_base_url')}/",
            $payment->webhookUrl
          );
        }

        // Send the updated webhook URL to Mollie.
        $payment->update();
      }

      // Allow other modules to react to the status change and alter the status
      // code returned.
      $event = new MollieTransactionStatusChangeEvent($context, $context_id, $transaction, $payment);
      $event->setHttpStatusCode(200);
      $this->eventDispatcher->dispatch($event, MollieTransactionStatusChangeEvent::EVENT_NAME);
    }
    catch (PluginNotFoundException | InvalidPluginDefinitionException | ApiException $e) {
      watchdog_exception('mollie', $e);
      return new Response('', 500);
    }

    $http_status_code = $event->getHttpStatusCode() ?: 500;
    return new Response('', $http_status_code);
  }

  /**
   * Allows modules to react to payment updates for paid payments.
   *
   * Mollie calls this webhook when:
   *  - a refund is performed on the payment,
   *  - a chargeback is performed on the payment.
   *
   * This code is experimental. The MollieTransactionRefundEvent and
   * MollieTransactionChargebackEvent should be used with care and be thoroughly
   * tested before use in production.
   *
   * @param string $context
   *   The type of context that requested the payment.
   * @param string $context_id
   *   The ID of the context that requested the payment.
   *
   * @return \Symfony\Component\HttpFoundation\Response
   *   Response that will inform Mollie whether the event was processed
   *   correctly.
   */
  public function invokeAftercareHook(string $context, string $context_id): Response {
    $transaction_id = $this->requestStack->getCurrentRequest()->get('id');

    // Register invocation of the webhook.
    $previous_invocation = $this->keyValue(Mollie::LAST_WEBHOOK_INVOCATION_COLLECTION_KEY)->get($transaction_id);
    $this->keyValue(Mollie::LAST_WEBHOOK_INVOCATION_COLLECTION_KEY)->set($transaction_id, date('c'));

    try {
      // Get the transaction for which the hook was invoked.
      /** @var \Drupal\mollie\TransactionInterface[] $transactions */
      $transactions = $this->entityTypeManager()->getStorage('mollie_payment')
        ->loadByProperties([
          'transaction_id' => $transaction_id,
          'context' => $context,
          'context_id' => $context_id,
        ]);
      // The transaction ID should be unique, so we can safely assume that there
      // is only one transaction with the given ID.
      $transaction = reset($transactions);

      if (!$transaction) {
        // Just return a 200 OK if the transaction could not be loaded. This is
        // advised by Mollie.
        return new Response('', 200);
      }

      $payment = $this->mollie->getClient()->payments->get($transaction->getTransactionId());

      // Loop over refunds and see if a new one has been added since the
      // previous webhook invocation.
      foreach ($payment->refunds() as $refund) {
        /** @var \Mollie\Api\Resources\Refund $refund */
        if ($refund->createdAt > $previous_invocation) {
          $event = new MollieTransactionRefundEvent($context, $context_id, $transaction, $payment);
          break;
        }
      }

      // If no new refund was found, check for chargebacks.
      if (!isset($event)) {
        foreach ($payment->chargebacks() as $chargeback) {
          /** @var \Mollie\Api\Resources\Chargeback $chargeback */
          if ($chargeback->createdAt > $previous_invocation) {
            $event = new MollieTransactionChargebackEvent($context, $context_id, $transaction, $payment);
            break;
          }
        }
      }

      if (isset($event)) {
        $event->setHttpStatusCode(200);
        $this->eventDispatcher->dispatch($event::EVENT_NAME, $event);
      }
    }
    catch (PluginNotFoundException | InvalidPluginDefinitionException | ApiException $e) {
      watchdog_exception('mollie', $e);
      return new Response('', 500);
    }

    // For now, we just return a 200 OK. We will add support for aftercare
    // events later.
    return new Response('', $event->getHttpStatusCode());
  }

}
