<?php

namespace Drupal\mollie\Entity;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Language\LanguageInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\Core\Url;
use Mollie\Api\Resources\Payment as MolliePayment;

/**
 * Class Payment.
 *
 * @package Drupal\mollie\Entity
 *
 * @ContentEntityType(
 *   id = "mollie_payment",
 *   label = @Translation("Mollie payment"),
 *   handlers = {
 *     "list_builder" = "Drupal\mollie\PaymentListBuilder",
 *     "form" = {
 *       "add" = "Drupal\mollie\Form\PaymentForm",
 *     },
 *     "access" = "Drupal\mollie\PaymentAccessControlHandler",
 *   },
 *   base_table = "mollie_payment",
 *   fieldable = FALSE,
 *   entity_keys = {
 *     "id" = "id",
 *     "label" = "transaction_id",
 *     "uuid" = "uuid",
 *   },
 *   links = {
 *     "canonical" = "/mollie/payment/{mollie_payment}",
 *     "collection" = "/admin/mollie/payments",
 *     "add-form" = "/mollie/payment/add",
 *   },
 * )
 */
class Payment extends TransactionBase {

  /**
   * @param array $payment_data
   *
   * @return \Drupal\mollie\Entity\Payment
   * @throws \Drupal\Core\TypedData\Exception\MissingDataException
   */
  public static function createFromPaymentData(array $payment_data): Payment {
    // Validate payment data.
    if (empty($payment_data['description'])) {
      throw new \InvalidArgumentException('Payment data is missing a description.');
    }
    if (empty($payment_data['amount'])) {
      throw new \InvalidArgumentException('Payment data is missing an amount.');
    }
    if (!is_string($payment_data['amount'])) {
      throw new \InvalidArgumentException('The amount should be passed in as a string.');
    }
    // Set (default) values.
    $description = $payment_data['description'];
    $amount = $payment_data['amount'];
    $currency = $payment_data['currency'] ?? 'EUR';
    $context = $payment_data['context'] ?? 'manual';
    $context_id = $payment_data['context_id'] ?? \Drupal::service('uuid')->generate();
    $method = $payment_data['method'] ?? '';
    $issuer = $payment_data['issuer'] ?? '';

    $redirectUrl = Url::fromRoute(
      'mollie.redirect',
      ['context' => $context, 'context_id' => $context_id]
    )->setAbsolute()->toString();
    $webhookUrl = Url::fromRoute(
      'mollie.webhook.status_change',
      ['context' => $context, 'context_id' => $context_id]
    )->setAbsolute()->toString();

    // Change webhook when using an alternative base URL in test mode.
    $config = \Drupal::config('mollie.config');
    if ($config->get('test_mode') && $config->get('webhook_base_url') !== '') {
      $defaultBaseUrl = Url::fromRoute('<front>')
        ->setAbsolute()->toString();
      $webhookUrl = str_replace(
        $defaultBaseUrl,
        "{$config->get('webhook_base_url')}/",
        $webhookUrl
      );
    }

    $metadata = [
      'context' => $context,
      'context_id' => $context_id,
    ];

    $values = [
      'description' => $description,
      'amount' => [
        'value' => $amount,
        'currency' => $currency,
      ],
      'method' => $method,
      'redirectUrl' => $redirectUrl,
      'webhookUrl' => $webhookUrl,
      'metadata' => Json::encode($metadata),
      'locale' => static::getLocaleByCurrentContentLanguage(),
    ];

    if (in_array($method, ['ideal']) && !empty($issuer)) {
      $values['issuer'] = $issuer;
    }

    // Create a payment on the Mollie side.
    $mollie_payment = \Drupal::service('mollie.mollie')->getClient()->payments->create($values);

    // Create and return a payment entity.
    $data = static::MolliePaymentToArray($mollie_payment);
    return static::create([
      'transaction_id' => $data['id'],
      'context' => $data['context'],
      'context_id' => $data['contextId'],
      'data' => $data,
    ]);
  }

  /**
   * {@inheritdoc}
   */
  public static function baseFieldDefinitions(EntityTypeInterface $entity_type) {
    /** @var \Drupal\Core\Field\BaseFieldDefinition[] $fields */
    $fields = parent::baseFieldDefinitions($entity_type);

    if (isset($fields['transaction_id'])) {
      $fields['transaction_id']->setDescription(new TranslatableMarkup('The ID of the Mollie payment.'));
    }

    if (isset($fields['data'])) {
      $fields['data']->setDescription(new TranslatableMarkup('The data related to the Mollie payment.'));
      $fields['data']->setDisplayOptions('view', [
        'label' => 'hidden',
        'type' => 'mollie_payment_data',
        'weight' => 0,
      ]);
    }

    return $fields;
  }

  /**
   * Returns the description for the payment.
   *
   * @return string
   *   Description for the payment.
   */
  public function getDescription(): string {
    return $this->getData()['description'] ?? '';
  }

  /**
   * Returns the refunded amount for the transaction.
   *
   * @return float
   *   Refunded amount for the transaction.
   */
  public function getRefundedAmount(): float {
    return $this->getData()['amountRefunded']->value ?? 0.0;
  }

  /**
   * Returns the currency of the refunded amount for the transaction.
   *
   * @return string
   *   Currency of the refunded amount for the transaction.
   */
  public function getRefundedCurrency(): string {
    return $this->getData()['amountRefunded']->currency ?? 'EUR';
  }

  /**
   * Returns the refundable amount for the transaction.
   *
   * @return float
   *   Refundable amount for the transaction.
   */
  public function getRefundableAmount(): float {
    return $this->getData()['amountRemaining']->value ?? 0.0;
  }

  /**
   * Returns the currency of the refundable amount for the transaction.
   *
   * @return string
   *   Currency of the refundable amount for the transaction.
   */
  public function getRefundableCurrency(): string {
    return $this->getData()['amountRemaining']->currency ?? 'EUR';
  }

  /**
   * Returns the captured amount for the transaction.
   *
   * @return float
   *   Captured amount for the transaction.
   */
  public function getCapturedAmount(): float {
    return $this->getData()['amountCaptured']->value ?? 0.0;
  }

  /**
   * Returns the currency of the captured amount for the transaction.
   *
   * @return string
   *   Currency of the captured amount for the transaction.
   */
  public function getCapturedCurrency(): string {
    return $this->getData()['amountCaptured']->currency ?? 'EUR';
  }

  /**
   * Returns the charged back amount for the transaction.
   *
   * @return float
   *   Charged back amount for the transaction.
   */
  public function getChargedBackAmount(): float {
    return $this->getData()['amountChargedBack']->value ?? 0.0;
  }

  /**
   * Returns the currency of the charged back amount for the transaction.
   *
   * @return string
   *   Currency of the charged back amount for the transaction.
   */
  public function getChargedBackCurrency(): string {
    return $this->getData()['amountChargedBack']->currency ?? 'EUR';
  }

  /**
   * Returns the payment method for the payment.
   *
   * @return string
   *   Payment method for the payment.
   */
  public function getMethod(): ?string {
    return $this->getData()['method'] ?? NULL;
  }

  /**
   * Returns the checkout URL for the payment.
   *
   * @return string
   *   Checkout URL for the payment.
   */
  public function getCheckoutUrl(): ?string {
    return $this->getData()['checkoutUrl'] ?? NULL;
  }

  /**
   * Turns a Mollie payment into an array containing the relevant data.
   *
   * @param \Mollie\Api\Resources\Payment $mollie_payment
   *   The Mollie payment.
   *
   * @return array
   *   The relevant data from the Mollie payment as an associative array.
   */
  public static function MolliePaymentToArray(MolliePayment $mollie_payment): array {
    // If the metadata property is an object, we encode it into a JSON string
    // for consistency.
    if (is_object($mollie_payment->metadata)) {
      $mollie_payment->metadata = Json::encode($mollie_payment->metadata);
    }
    // If the metadata property is not an array already, we assume it is a
    // JSON string and decode it.
    $metadata = is_array($mollie_payment->metadata)
      ? $mollie_payment->metadata
      : Json::decode($mollie_payment->metadata ?? '{}');

    $values = [
      'id' => $mollie_payment->id,
      'createdAt' => $mollie_payment->createdAt,
      'authorizedAt' => $mollie_payment->authorizedAt,
      'paidAt' => $mollie_payment->paidAt,
      'canceledAt' => $mollie_payment->canceledAt,
      'expiredAt' => $mollie_payment->expiredAt,
      'failedAt' => $mollie_payment->failedAt,
      'status' => $mollie_payment->status,
      'amount' => $mollie_payment->amount,
      'amountRefunded' => $mollie_payment->amountRefunded ?? NULL,
      'amountRemaining' => $mollie_payment->amountRemaining ?? NULL,
      'amountCaptured' => $mollie_payment->amountCaptured ?? NULL,
      'amountChargedBack' => $mollie_payment->amountChargedBack ?? NULL,
      'description' => $mollie_payment->description,
      'mode' => $mollie_payment->mode,
      'method' => $mollie_payment->method,
      'context' => $metadata['context'] ?? '',
      'contextId' => $metadata['context_id'] ?? '',
    ];
    if (isset($metadata['issuer'])) {
      $values['issuer'] = $metadata['issuer'];
    }
    if (isset($mollie_payment->_links->checkout)) {
      $values['checkoutUrl'] = $mollie_payment->_links->checkout->href;
    }

    return $values;
  }

  /**
   * Returns an ISO locale code based on the current content language.
   *
   * Best effort to get a locale code to pass to Mollie. Drupal does not have
   * a country for (anonymous) visitors by default so we cannot construct the
   * locale from language code and country code. This implementation is based
   * on the locales currently supported by Mollie.
   *
   * @return string
   */
  protected static function getLocaleByCurrentContentLanguage(): string {
    $langcode = \Drupal::languageManager()
      ->getCurrentLanguage(LanguageInterface::TYPE_CONTENT)
      ->getId();

    switch ($langcode) {
      case 'en':
        return 'en_US';

      case 'ca':
        return 'ca_ES';

      case 'nb':
        return 'nb_NO';

      case 'sv':
        return 'sv_SE';

      case 'da':
        return 'da_DK';

      default:
        $countryCode = strtoupper($langcode);
        return "{$langcode}_{$countryCode}";
    }
  }

}
