<?php

namespace Drupal\mollie\Entity;

use DateTimeInterface;
use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\Core\Entity\ContentEntityBase;
use Drupal\Core\Entity\EntityChangedTrait;
use Drupal\Core\Entity\EntityStorageException;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\mollie\TransactionInterface;

/**
 * Class TransactionBase.
 *
 * @package Drupal\mollie\Entity
 */
abstract class TransactionBase extends ContentEntityBase implements TransactionInterface {

  use EntityChangedTrait;

  /**
   * {@inheritdoc}
   */
  public static function baseFieldDefinitions(EntityTypeInterface $entity_type) {
    $fields = parent::baseFieldDefinitions($entity_type);

    // Transaction ID.
    $fields['transaction_id'] = BaseFieldDefinition::create('string')
      ->setLabel(new TranslatableMarkup('Transaction ID'))
      ->setReadOnly(TRUE)
      ->addConstraint('UniqueField');

    // Transaction context.
    $fields['context'] = BaseFieldDefinition::create('string')
      ->setLabel(new TranslatableMarkup('Context'))
      ->setDescription(new TranslatableMarkup('Type of context that requested the transaction.'));

    // Transaction context ID.
    $fields['context_id'] = BaseFieldDefinition::create('string')
      ->setLabel(new TranslatableMarkup('Context ID'))
      ->setDescription(new TranslatableMarkup('ID of the context that requested the transaction.'));

    // Transaction data.
    $fields['data'] = BaseFieldDefinition::create('map')
      ->setLabel(new TranslatableMarkup('Data'));

    return $fields;
  }

  /**
   * {@inheritdoc}
   */
  public function getTransactionId(): string {
    return $this->get('transaction_id')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function getCreatedTime(): string {
    return $this->getData()['createdAt'] ?? '';
  }

  /**
   * {@inheritdoc}
   */
  public function getStatus(): string {
    return $this->getData()['status'] ?? '';
  }

  /**
   * {@inheritdoc}
   */
  public function getAmount(): float {
    return $this->getData()['amount']->value ?? 0.0;
  }

  /**
   * {@inheritdoc}
   */
  public function getCurrency(): string {
    return $this->getData()['amount']->currency ?? 'EUR';
  }

  /**
   * {@inheritdoc}
   */
  public function getMode(): string {
    return $this->getData()['mode'] ?? '';
  }

  /**
   * {@inheritdoc}
   */
  public function getContext(): string {
    return $this->get('context')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function getContextId(): string {
    return $this->get('context_id')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function getChangedTime() {
    // @TODO We need to return the newest of all time fields.
    $changed = $this->getData()['createdAt'];

    $data = $this->getData();
    $dateFields = [
      'authorizedAt',
      'paidAt',
      'canceledAt',
      'expiredAt',
      'failedAt',
    ];
    foreach ($dateFields as $dateField) {
      if (array_key_exists($dateField, $data) && $data[$dateField] > $changed) {
        $changed = $data[$dateField];
      }
    }

    return DrupalDateTime::createFromFormat(DateTimeInterface::ATOM, $changed)->getTimestamp();
  }

  /**
   * {@inheritdoc}
   */
  public function setChangedTime($timestamp) {
    throw new EntityStorageException('The storage of the mollie_payment entity type does not allow updating the changed time.');
  }

  /**
   * Returns the data associated with this transaction as on array.
   *
   * @return array
   *   Associative array of transaction data.
   */
  protected function getData(): array {
    return $this->get('data')->getValue()[0];
  }

}
