<?php

namespace Drupal\mollie\Form;

use Drupal\Core\Entity\ContentEntityForm;
use Drupal\Core\Entity\EntityStorageException;
use Drupal\Core\Form\FormStateInterface;
use Drupal\mollie\Entity\Payment;
use Mollie\Api\Exceptions\ApiException;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a form for adding mollie_payment entities.
 */
class PaymentForm extends ContentEntityForm {

  /**
   * The Mollie API client.
   *
   * @var \Drupal\mollie\Mollie
   */
  protected $mollieApiClient;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = new static(
      $container->get('entity.repository'),
      $container->get('entity_type.bundle.info'),
      $container->get('datetime.time')
    );

    $instance->setMessenger($container->get('messenger'));
    $instance->setStringTranslation($container->get('string_translation'));

    $instance->mollieApiClient = $container->get('mollie.mollie');

    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {
    $form['description'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Description'),
      '#placeholder' => $this->t('Shortly describe what this payment is for.'),
    ];

    $form['currency'] = [
      '#type' => 'select',
      '#title' => $this->t('Currency'),
      '#options' => [
        // @todo Multiple currencies.
        'EUR' => $this->t('Euro'),
      ],
    ];

    $form['amount'] = [
      '#type' => 'number',
      '#title' => $this->t('Amount'),
      '#step' => '0.01',
      // Ajaxify the amount field.
      '#ajax' => [
        'callback' => [static::class, 'updateMethods'],
        'wrapper' => 'available-payment-methods',
        // Prevent autofocus on the field after ajax call. This can probably be
        // removed once #2627788 lands.
        // @see https://www.drupal.org/project/drupal/issues/2627788
        'disable-refocus' => TRUE,
      ]
    ];

    $form['method'] = [
      '#type' => 'select',
      '#title' => $this->t('Payment method'),
      '#empty_option' => $this->t('Select a payment method'),
      // Wrap the method field for ajaxification.
      '#prefix' => '<div id="available-payment-methods">',
      '#suffix' => '</div>',
    ];

    // Get the available payment methods if we have an amount and currency.
    if ($form_state->hasValue('amount') && $form_state->hasValue('currency')) {
      $form['method']['#options'] = $this->mollieApiClient->getMethods(
        (float) $form_state->getValue('amount'),
        $form_state->getValue('currency')
      );
    }

    return parent::form($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    try {
      // Create a payment.
      $payment = Payment::createFromPaymentData([
        'description' => $form_state->getValue('description'),
        'amount' => number_format($form_state->getValue('amount'), 2, '.', ''),
        'currency' => $form_state->getValue('currency'),
        'context' => 'form',
        'method' => $form_state->getValue('method') ?? '',
        'issuer' => $form_state->getValue('issuer') ?? '',
      ]);
      // Store the payment entity and let the rest of the entity form do its
      // thing.
      $this->entity = $payment;

      // Return to the payment overview.
      $form_state->setRedirect('entity.mollie_payment.collection');
    } catch (ApiException | EntityStorageException $e) {
      watchdog_exception('mollie', $e);
      $this->messenger()->addError($this->t('Something went wrong while creating the payment. Please try again later.'));
    }

  }

  /**
   * Returns a renderable array for the method form element.
   *
   * @param $form
   *   Renderable array for the form as build by the form builder.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   Current form state.
   *
   * @return array
   *   Renderable array for the method form element.
   */
  public static function updateMethods($form, FormStateInterface $form_state): array {
    return $form['method'];
  }

}
