<?php

namespace Drupal\mollie;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Messenger\MessengerTrait;
use Drupal\Core\Site\Settings;
use Drupal\Core\Url;
use Mollie\Api\Exceptions\ApiException;
use Mollie\Api\Exceptions\IncompatiblePlatform;
use Mollie\Api\MollieApiClient;
use Mollie\Api\Resources\MethodCollection;
use Mollie\Api\Resources\Payment as MolliePayment;

/**
 * Class Mollie.
 *
 * @package Drupal\mollie
 */
class Mollie {

  use MessengerTrait;

  /**
   * Key of collection of last webhook invocation dates in the key/value store.
   */
  const LAST_WEBHOOK_INVOCATION_COLLECTION_KEY = 'mollie_last_webhook_invocation';

  /**
   * The configuration factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The Mollie config validator.
   *
   * @var \Drupal\mollie\MollieConfigValidator
   */
  protected $configValidator;

  /**
   * Mollie API client.
   *
   * @var \Mollie\Api\MollieApiClient
   */
  protected $client;

  /**
   * Constructs a new Mollie object.
   *
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The configuration factory.
   * @param \Drupal\mollie\MollieConfigValidator $configValidator
   *   The Mollie config validator.
   */
  public function __construct(MessengerInterface $messenger, ConfigFactoryInterface $config_factory, MollieConfigValidator $config_validator) {
    $this->setMessenger($messenger);
    $this->configFactory = $config_factory;
    $this->configValidator = $config_validator;
  }

  /**
   * Returns a client for communication with the Mollie API.
   *
   * @return \Mollie\Api\MollieApiClient|null
   *   A client for communication with the Mollie API or NULL if this client
   *   could not be loaded.
   */
  public function getClient(): ?MollieApiClient {
    // Static caching.
    if (isset($this->client)) {
      return $this->client;
    }

    try {
      $client = new MollieApiClient();

      // Add version strings. These are used by Mollie to gather statistics
      // about platform usage.
      $client->addVersionString('Drupal/' . \Drupal::VERSION);

      if ($this->useTestMode()) {
        $client->setApiKey(Settings::get('mollie.settings')['test_key']);
      }
      elseif ($this->configValidator->hasLiveApiKey()) {
        $client->setApiKey(Settings::get('mollie.settings')['live_key']);
      }

      $this->client = $client;
      return $this->client;
    }
    catch (IncompatiblePlatform $e) {
      watchdog_exception('mollie', $e);
      $this->messenger()->addError(t('This project is not compatible with Mollie API client for PHP.'));
    }
    catch (ApiException $e) {
      watchdog_exception('mollie', $e);
      $this->messenger()->addError(t('The Mollie API client for PHP could not be initialized.'));
    }

    return NULL;
  }

  /**
   * Creates a Mollie payment.
   *
   * @todo Shouldn't this be part of the entity?
   *
   * @param string $description
   *   The description of the payment.
   * @param string $amount
   *   The amount of the payment as a string.
   * @param string $currency
   *   (optional) The currency of the payment.
   * @param string $context
   *   (optional) The context of the payment.
   * @param string $context_id
   *   (optional) The context ID of the payment.
   * @param string $method
   *   (optional) The payment method.
   * @param string $issuer
   *   (optional) The payment issuer.
   *
   * @return \Mollie\Api\Resources\Payment
   *   The Mollie payment object.
   *
   * @throws \Mollie\Api\Exceptions\ApiException
   */
  public function createMolliePayment(string $description, string $amount, string $currency = 'EUR', string $context = 'manual', string $context_id = '', string $method = '', string $issuer = ''): MolliePayment {
    // Generate a context ID if we do not have one.
    if (empty($context_id)) {
      $context_id = $this->uuid->generate();
    }

    $redirectUrl = Url::fromRoute(
      'mollie.redirect',
      ['context' => $context, 'context_id' => $context_id]
    )->setAbsolute()->toString();
    $webhookUrl = Url::fromRoute(
      'mollie.webhook.status_change',
      ['context' => $context, 'context_id' => $context_id]
    )->setAbsolute()->toString();

    // Change webhook when using an alternative base URL in test mode.
    $config = $this->configFactory->get('mollie.config');
    if ($config->get('test_mode') && $config->get('webhook_base_url') !== '') {
      $defaultBaseUrl = Url::fromRoute('<front>')
        ->setAbsolute()->toString();
      $webhookUrl = str_replace(
        $defaultBaseUrl,
        "{$config->get('webhook_base_url')}/",
        $webhookUrl
      );
    }

    $metadata = [
      'context' => $context,
      'context_id' => $context_id,
    ];

    $values = [
      'description' => $description,
      'amount' => [
        'value' => $amount,
        'currency' => $currency,
      ],
      'method' => $method,
      'redirectUrl' => $redirectUrl,
      'webhookUrl' => $webhookUrl,
      'metadata' => Json::encode($metadata),
      'locale' => $this->getLocaleByCurrentContentLanguage(),
    ];

    if (in_array($method, ['ideal']) && !empty($issuer)) {
      $values['issuer'] = $issuer;
    }

    // Create a payment on the Mollie side.
    return $this->getClient()->payments->create($values);
  }

  /**
   * Determines whether test mode should be used.
   *
   * @return bool
   *   True if test mode should be used, false otherwise.
   */
  public function useTestMode(): bool {
    return $this->configValidator->hasTestApiKey()
      && $this->configFactory->get('mollie.config')->get('test_mode');
  }

  /**
   * Returns an array of payment methods available for a given amount.
   *
   * @param float $amount
   *   Amount to be paid.
   * @param string $currency
   *   Currency in which the amount should be paid.
   *
   * @return array
   *   Associative array, keyed by payment method ID.
   */
  public function getMethods(float $amount, string $currency): array {
    $methods = [];

    foreach ($this->getMethodsRaw($amount, $currency) as $method) {
      /** @var \Mollie\Api\Resources\Method $method */
      $methods[$method->id] = $method->description;
    }

    return $methods;
  }

  /**
   * Returns the available payment statuses.
   *
   * @return array
   *   Associative array keyed by payment status ID.
   */
  public function getStatuses(): array {
    return [
      'opem' => t('Open'),
      'cancelled' => t('Cancelled'),
      'pending' => t('Pending'),
      'authorized' => t('Authorized'),
      'expired' => t('Expired'),
      'failed' => t('Failed'),
      'paid' => t('Paid'),
    ];
  }

  /**
   * Returns a collection of payment methods available for a given amount.
   *
   * @param float $amount
   *   Amount to be paid.
   * @param string $currency
   *   Currency in which the amount should be paid.
   *
   * @return \Mollie\Api\Resources\MethodCollection
   *   Method collection.
   */
  protected function getMethodsRaw(float $amount, string $currency): MethodCollection {
    try {
      return $this->getClient()->methods->allActive(
        [
          'amount' => [
            'value' => number_format($amount, 2, '.', ''),
            'currency' => $currency,
          ]
        ]
      );
    }
    catch (ApiException $e) {
      watchdog_exception('mollie', $e);
    }

    return new MethodCollection(0, NULL);
  }

}
