<?php

namespace Drupal\mollie;

use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityListBuilder;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Link;
use Drupal\Core\TypedData\Type\DateTimeInterface;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a list builder for the mollie_payment entity.
 */
class PaymentListBuilder extends EntityListBuilder {

  /**
   * Date formatter.
   *
   * @var \Drupal\Core\Datetime\DateFormatterInterface
   */
  protected $dateFormatter;

  /**
   * Constructs a new EntityListBuilder object.
   *
   * @param \Drupal\Core\Entity\EntityTypeInterface $entityType
   *   The entity type definition.
   * @param \Drupal\Core\Entity\EntityStorageInterface $storage
   *   The entity storage class.
   * @param \Drupal\Core\Datetime\DateFormatterInterface $dateFormatter
   *   Date formatter.
   */
  public function __construct(
    EntityTypeInterface $entityType,
    EntityStorageInterface $storage,
    DateFormatterInterface $dateFormatter
  ) {
    parent::__construct($entityType, $storage);

    $this->dateFormatter = $dateFormatter;
  }

  /**
   * {@inheritdoc}
   */
  public static function createInstance(ContainerInterface $container, EntityTypeInterface $entityType) {
    return new static(
      $entityType,
      $container->get('entity_type.manager')->getStorage($entityType->id()),
      $container->get('date.formatter')
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function getEntityIds() {
    // We override this method to sort the list by the entity ID in descending
    // order.
    $query = $this->getStorage()->getQuery()
      ->accessCheck()
      ->sort($this->entityType->getKey('id'), 'DESC');

    // Only add the pager if a limit is specified.
    if ($this->limit) {
      $query->pager($this->limit);
    }
    return $query->execute();
  }

  /**
   * {@inheritdoc}
   */
  public function buildHeader() {
    $header = [];

    $header['id'] = [
      'data' => $this->t('ID'),
      'class' => [RESPONSIVE_PRIORITY_MEDIUM],
    ];
    $header['transaction_id'] = $this->t('Transaction ID');
    $header['description'] = [
      'data' => $this->t('Description'),
      'class' => [RESPONSIVE_PRIORITY_MEDIUM],
    ];
    $header['mode'] = [
      'data' => $this->t('Mode'),
      'class' => [RESPONSIVE_PRIORITY_MEDIUM],
    ];
    $header['amount'] = $this->t('Amount');
    $header['refunded_amount'] = [
      'data' => $this->t('Refunded amount'),
      'class' => [RESPONSIVE_PRIORITY_LOW],
    ];
    $header['refundable_amount'] = [
      'data' => $this->t('Refundable amount'),
      'class' => [RESPONSIVE_PRIORITY_LOW],
    ];
    $header['captured_amount'] = [
      'data' => $this->t('Captured mount'),
      'class' => [RESPONSIVE_PRIORITY_LOW],
    ];
    $header['charged_back_amount'] = [
      'data' => $this->t('Charged back amount'),
      'class' => [RESPONSIVE_PRIORITY_LOW],
    ];
    $header['status'] = $this->t('Status');
    $header['created'] = [
      'data' => $this->t('Created'),
      'class' => [RESPONSIVE_PRIORITY_LOW],
    ];
    $header['changed'] = [
      'data' => $this->t('Changed'),
      'class' => [RESPONSIVE_PRIORITY_LOW],
    ];

    return $header + parent::buildHeader();
  }

  /**
   * {@inheritdoc}
   */
  public function buildRow(EntityInterface $entity) {
    /** @var \Drupal\mollie\Entity\Payment $entity */
    $row = [];

    $row['id'] = $entity->id();
    $row['transaction_id'] = Link::fromTextAndUrl($entity->getTransactionId(), $entity->toUrl());
    $row['description'] = $entity->getDescription();
    $row['mode'] = $entity->getMode();
    $row['amount'] = $this->t(
      '@amount @currency',
      [
        '@amount' => $entity->getAmount(),
        '@currency' => $entity->getCurrency(),
      ]
    );
    $row['refunded_amount'] = $this->t(
      '@amount @currency',
      [
        '@amount' => $entity->getRefundedAmount(),
        '@currency' => $entity->getRefundedCurrency(),
      ]
    );
    $row['refundable_amount'] = $this->t(
      '@amount @currency',
      [
        '@amount' => $entity->getRefundableAmount(),
        '@currency' => $entity->getRefundableCurrency(),
      ]
    );
    $row['captured_amount'] = $this->t(
      '@amount @currency',
      [
        '@amount' => $entity->getCapturedAmount(),
        '@currency' => $entity->getCapturedCurrency(),
      ]
    );
    $row['charged_back_amount'] = $this->t(
      '@amount @currency',
      [
        '@amount' => $entity->getChargedBackAmount(),
        '@currency' => $entity->getChargedBackCurrency(),
      ]
    );
    $row['status'] = $entity->getStatus() === 'open'
      ? Link::fromTextAndUrl($entity->getStatus(), Url::fromUri($entity->getCheckoutUrl()))
      : $entity->getStatus();
    $row['created'] = $this->dateFormatter->format(DrupalDateTime::createFromFormat(\DateTimeInterface::ATOM, $entity->getCreatedTime())->getTimestamp());
    $row['changed'] = $this->dateFormatter->format($entity->getChangedTime());

    return $row + parent::buildRow($entity);
  }

}
